@@ -4,8 +4,8 @@ use serde::Serialize;
 
 #[derive(Default, Copy, Clone, Debug, PartialEq, Serialize, Deserialize)]
 pub struct BoundingSphere {
-    pub radius: f32,
     pub position: Vec3,
+    pub radius: f32,
 }
 
 impl BoundingSphere {
@@ -13,6 +13,6 @@ impl BoundingSphere {
         position: Vec3,
         radius: f32,
     ) -> Self {
-        BoundingSphere { radius, position }
+        BoundingSphere { position, radius }
     }
 }
