@@ -3,6 +3,7 @@ mod internal;
 mod polygon_soup;
 mod projection;
 mod view_frustum;
+mod visibility_object;
 mod visibility_world;
 mod visible_bounds;
 
@@ -11,5 +12,6 @@ pub mod geometry;
 pub use polygon_soup::*;
 pub use projection::*;
 pub use view_frustum::*;
+pub use visibility_object::*;
 pub use visibility_world::*;
 pub use visible_bounds::*;
